#include <TFT_eSPI.h>
#include <CST816S.h>
#include "Free_Fonts.h"
#include <Wire.h>

#include "USB.h"
#include "USBHIDKeyboard.h"
USBHIDKeyboard Keyboard;

TFT_eSPI tft = TFT_eSPI();
CST816S touch(5, 6, 10, 7);

String keys[12] = {"1", "2", "3", "4", "5", "6", "7", "8", "9", "*", "0", "#"};
String password = "1234"; // Set your password here
String inputPassword = "";
bool passwordEntered = false;
bool keyPressed[12] = {false}; // Flag to track if a key is pressed

enum Screen { PASSWORD_SCREEN, OPTIONS_SCREEN };
Screen currentScreen = PASSWORD_SCREEN;

unsigned long optionsScreenStartTime = 0;
const unsigned long optionsScreenDelay = 1000; // 40 seconds delay

void setup() {
  Serial.begin(115200);
  Keyboard.begin();
  USB.begin();

  pinMode(TFT_BL, OUTPUT);
  digitalWrite(TFT_BL, HIGH);

  tft.begin();
  tft.fillScreen(TFT_BLACK);
  tft.setTextColor(TFT_WHITE, TFT_BLACK);
  tft.setFreeFont(FSB9);

  tft.setCursor(30, 45); // Adjusted cursor position for typed string
  tft.print(inputPassword + "_");
  touch.begin();

  drawKeypad();
}

void drawKeypad() {
  int keyWidth = 36; // Increased key width
  int keyHeight = 36; // Increased key height
  int startX = 63; // Adjusted startX
  int startY = 78; // Moved keypad up by 20 pixels

  int col = 0;
  int row = 0;

  for (int i = 0; i < 12; i++) {
    int x = startX + (col * (keyWidth + 5));
    int y = startY + (row * (keyHeight + 5));

    tft.drawRoundRect(x, y, keyWidth, keyHeight, 5, TFT_WHITE);
    tft.setCursor(x + 6, y + 12); // Adjusted cursor position for text inside box

    // Approximating 1.5 pixels down by rounding to the nearest whole pixel
    tft.print(keys[i]);
    
    col++;
    if (col == 3) {
      col = 0;
      row++;
    }
  }
}

void drawOptions() {
  tft.fillScreen(TFT_BLACK);
  tft.setCursor(30, 40); // Adjusted cursor position for text
  tft.print("Options Screen");
  
  // Draw touch options
  // You can customize the positions and appearance of these buttons
  tft.drawRoundRect(50, 80, 100, 50, 5, TFT_WHITE);
  tft.setCursor(70, 100);
  tft.print("Laptop");
  
  tft.drawRoundRect(50, 140, 100, 50, 5, TFT_WHITE);
  tft.setCursor(70, 160);
  tft.print("Key");
  
  tft.drawRoundRect(50, 200, 100, 50, 5, TFT_WHITE);
  tft.setCursor(70, 220);
  tft.print("UPI");

  optionsScreenStartTime = millis(); // Record the start time for options screen
}

void displayWrongPassword() {
  tft.fillScreen(TFT_BLACK);
  tft.setCursor(30, 40); // Adjusted cursor position for text
  tft.setTextColor(TFT_RED); // Set text color to red
  tft.print("Wrong Password!");
  delay(1000);
  tft.fillScreen(TFT_BLACK);
  drawKeypad();
  inputPassword = "";
  tft.setCursor(30, 45); // Adjusted cursor position for typed string
  tft.setTextColor(TFT_WHITE); // Reset text color to white
  tft.print(inputPassword + "_");
}

void executeLaptopFunction() {
  // Functionality for the Laptop option
  // For demonstration, let's just send the password as HID input
   Keyboard.print("efy123");
  tft.fillScreen(TFT_BLACK);
  tft.setCursor(30, 40); // Adjusted cursor position for text
  tft.print("Password sent for Laptop:");
  drawKeypad();
  inputPassword = ""; // Clear the input password
  currentScreen = PASSWORD_SCREEN; // Go back to the password screen
}

void executeKeyFunction() {
  Keyboard.print("password");
   tft.fillScreen(TFT_BLACK);
  tft.setCursor(30, 40); // Adjusted cursor position for text
  tft.print("Password sent for door key:");
  tft.setCursor(30, 60);
  drawKeypad();
  inputPassword = ""; // Clear the input password
  currentScreen = PASSWORD_SCREEN; // Go back to the password screen
  // Functionality for the Key option
  // You can implement the desired functionality here
}

void executeUPIFunction() {
  Keyboard.print("8890");
   tft.fillScreen(TFT_BLACK);
  tft.setCursor(30, 40); // Adjusted cursor position for text
  tft.print("Password sent for UPI:");
  tft.fillScreen(TFT_BLACK);
   drawKeypad();
  inputPassword = ""; // Clear the input password
  currentScreen = PASSWORD_SCREEN; // Go back to the password screen
  // Functionality for the UPI option
  // You can implement the desired functionality here
}

void loop() {
  if (currentScreen == PASSWORD_SCREEN) {
    if (touch.available()) {
      int x = touch.data.x;
      int y = touch.data.y;
      String key = getKey(x, y);
      if (!keyPressed[key.toInt()]) { // Check if the key is not already pressed
        inputPassword += key;
        tft.setCursor(30, 45); // Adjusted cursor position for typed string
        tft.print(inputPassword + "_");
        keyPressed[key.toInt()] = true; // Set the flag for the pressed key
        if (inputPassword.length() == password.length()) {
          if (inputPassword == password) {
            passwordEntered = true;
            currentScreen = OPTIONS_SCREEN;
            drawOptions();
          } else {
            displayWrongPassword(); // Display wrong password message
          }
        }
      }
    }
  } else if (currentScreen == OPTIONS_SCREEN) {
    // Check if 40 seconds have passed since options screen started
    if (millis() - optionsScreenStartTime >= optionsScreenDelay) {
      if (touch.available()) {
        int x = touch.data.x;
        int y = touch.data.y;
        if (x > 50 && x < 150 && y > 80 && y < 130) {
          executeLaptopFunction();
        } else if (x > 50 && x < 150 && y > 140 && y < 190) {
          executeKeyFunction();
        } else if (x > 50 && x < 150 && y > 200 && y < 250) {
          executeUPIFunction();
        }
      }
    }
  }
}

String getKey(int x, int y) {
  int keyWidth = 36; // Increased key width
  int keyHeight = 36; // Increased key height
  int startX = 60; // Adjusted startX
  int startY = 78; // Moved keypad up by 20 pixels

  int col = (x - startX) / (keyWidth + 5);
  int row = (y - startY) / (keyHeight + 5);

  int index = row * 3 + col;
  if (index >= 0 && index < 12) {
    return keys[index];
  } else {
    return "";
  }
}
